# -*- coding: utf-8 -*-

#  PycaWM
#  misc.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('misc module needs at least Python 2.5!')

from Xlib import X, XK

modifiers = dict(C=X.ControlMask, M=X.Mod1Mask, S=X.ShiftMask)

modifier_indexes = {X.ControlMask: X.ControlMapIndex,
                    X.Mod1Mask: X.Mod1MapIndex,
                    X.ShiftMask: X.ShiftMapIndex}

def good_key(shortcut):
    global modifiers

    if not shortcut:
        return (X.NoSymbol, 0)

    keys = shortcut.split('-')
    mask = 0
    for modifier in keys[:-1]:
        try:
            mask |= modifiers[modifier.upper()]
        except KeyError:
            pass
    key = XK.string_to_keysym(keys[-1])
    return (key, mask)

numlock_mask = 0

def find_numlock_mask(display):
    global numlock_mask
    numlock_keycode = display.keysym_to_keycode(XK.XK_Num_Lock)

    for i, mask in enumerate(display.get_modifier_mapping()):
        if mask[0] == numlock_keycode:
            numlock_mask = 1 << i

def good_mask(mask):
    return mask & ~(numlock_mask | X.LockMask)

def good_grab_key(window, keycode, mask):
    global numlock_mask

    for mask_ in [mask, mask | numlock_mask, mask \
                      | X.LockMask, mask | numlock_mask | X.LockMask]:
        window.grab_key(keycode, mask_, 1, X.GrabModeAsync, X.GrabModeAsync)

def good_ungrab_key(window, keycode, mask):
    global numlock_mask

    for mask_ in [mask, mask | numlock_mask, mask \
                      | X.LockMask, mask | numlock_mask | X.LockMask]:
        window.ungrab_key(keycode, mask_)
